Files
derek.holloway 7502bf64a4
CodeQL / Analyze (actions) (push) Canceled after 0s
CodeQL / Analyze (python) (push) Canceled after 0s
pre-commit / pre-commit (push) Canceled after 0s
Multi-Arch CI / setup (push) Canceled after 0s
Publish build_manylinux_x86_64 images / publish_build_manylinux_x86_64 (push) Canceled after 0s
Setup (Multi-Arch) / setup (push) Canceled after 0s
Multi-Arch CI / Linux::${{ fromJSON(needs.setup.outputs.linux_build_config || '{}').build_variant_label || 'skip' }} (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Publish no_rocm_image_ubi10 image / publish_no_rocm_image_ubi10 (push) Canceled after 0s
Multi-Arch CI / Windows::${{ fromJSON(needs.setup.outputs.windows_build_config || '{}').build_variant_label || 'skip' }} (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Multi-Arch CI / Manifest Diff (push) Canceled after 0s
TheRock Manifest Diff Report / Generate Manifest Diff Report (push) Canceled after 0s
Multi-Arch CI / CI Summary (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04 (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04_ocl_rt images / publish_no_rocm_image_ubuntu24_04_ocl_rt (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_rocgdb (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04_rocgdb images / publish_no_rocm_image_ubuntu24_04_rocgdb (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_ocl_rt (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Unit Tests / Unit Tests :: ubuntu-24.04 (push) Canceled after 0s
Unit Tests / Unit Tests :: windows-2022 (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_media (push) Canceled after 0s
Unit Tests / Unit Tests Summary (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_openmpi (push) Canceled after 0s
Flatten subprojects into a single base monorepo
2026-09-02 16:34:47 -07:00

115 lines
3.7 KiB
C++

//===----------------------------------------------------------------------===//
//
// Part of the LLVM Project, under the Apache License v2.0 with LLVM Exceptions.
// See https://llvm.org/LICENSE.txt for license information.
// SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
// SPDX-FileCopyrightText: Copyright (c) 2024 NVIDIA CORPORATION & AFFILIATES.
//
//===----------------------------------------------------------------------===//
#ifndef LIBCXX_TEST_SUPPORT_TEST_RANGE_H
#define LIBCXX_TEST_SUPPORT_TEST_RANGE_H
#include <cuda/std/iterator>
#include <cuda/std/ranges>
#include "test_iterators.h"
struct sentinel
{
template <class I, cuda::std::enable_if_t<cuda::std::input_or_output_iterator<I>, int> = 0>
__host__ __device__ friend bool operator==(sentinel const&, I const&)
{
return true;
}
template <class I, cuda::std::enable_if_t<cuda::std::input_or_output_iterator<I>, int> = 0>
__host__ __device__ friend bool operator==(I const&, sentinel const&)
{
return true;
}
template <class I, cuda::std::enable_if_t<cuda::std::input_or_output_iterator<I>, int> = 0>
__host__ __device__ friend bool operator!=(sentinel const&, I const&)
{
return false;
}
template <class I, cuda::std::enable_if_t<cuda::std::input_or_output_iterator<I>, int> = 0>
__host__ __device__ friend bool operator!=(I const&, sentinel const&)
{
return false;
}
};
template <template <class...> class I, cuda::std::enable_if_t<cuda::std::input_or_output_iterator<I<int*>>, int> = 0>
struct test_range
{
__host__ __device__ I<int*> begin();
__host__ __device__ I<int const*> begin() const;
__host__ __device__ sentinel end();
__host__ __device__ sentinel end() const;
};
template <template <class...> class I, cuda::std::enable_if_t<cuda::std::input_or_output_iterator<I<int*>>, int> = 0>
struct test_non_const_range
{
__host__ __device__ I<int*> begin();
__host__ __device__ sentinel end();
};
template <template <class...> class I, cuda::std::enable_if_t<cuda::std::input_or_output_iterator<I<int*>>, int> = 0>
struct test_common_range
{
__host__ __device__ I<int*> begin();
__host__ __device__ I<int const*> begin() const;
__host__ __device__ I<int*> end();
__host__ __device__ I<int const*> end() const;
};
template <template <class...> class I, cuda::std::enable_if_t<cuda::std::input_or_output_iterator<I<int*>>, int> = 0>
struct test_non_const_common_range
{
__host__ __device__ I<int*> begin();
__host__ __device__ I<int*> end();
};
template <template <class...> class I, cuda::std::enable_if_t<cuda::std::input_or_output_iterator<I<int*>>, int> = 0>
struct test_view : cuda::std::ranges::view_base
{
__host__ __device__ I<int*> begin();
__host__ __device__ I<int const*> begin() const;
__host__ __device__ sentinel end();
__host__ __device__ sentinel end() const;
};
struct BorrowedRange
{
__host__ __device__ int* begin() const;
__host__ __device__ int* end() const;
__host__ __device__ BorrowedRange(BorrowedRange&&) = delete;
};
namespace cuda
{
namespace std
{
namespace ranges
{
template <>
_CCCL_INLINE_VAR constexpr bool enable_borrowed_range<BorrowedRange> = true;
} // namespace ranges
} // namespace std
} // namespace cuda
static_assert(!cuda::std::ranges::view<BorrowedRange>, "");
static_assert(cuda::std::ranges::borrowed_range<BorrowedRange>, "");
using BorrowedView = cuda::std::ranges::empty_view<int>;
static_assert(cuda::std::ranges::view<BorrowedView>, "");
static_assert(cuda::std::ranges::borrowed_range<BorrowedView>, "");
#if _LIBCUDACXX_HAS_RANGES
using NonBorrowedView = cuda::std::ranges::single_view<int>;
static_assert(cuda::std::ranges::view<NonBorrowedView>, "");
static_assert(!cuda::std::ranges::borrowed_range<NonBorrowedView>, "");
#endif // _LIBCUDACXX_HAS_RANGES
#endif // LIBCXX_TEST_SUPPORT_TEST_RANGE_H